#---------------------------
# Generating random numbers from distributions
# rnorm generates random values from the normal distribution
# rnorm(n, mu, sigma) gives us n random values from Norm(mu, sigma^2)
rnorm(5,0,1)     # Simulate 5 trials with a normal variable with mu=0, sigma=1 
rnorm(5,100,15)  # 5 trials with mu=100, sigma=15 (so sigma^2 = 225)

# Likewise for the exponential distribution
rexp(5,1)     # Simulate 5 trials with an exponential(1) variable
rexp(5,2.5)

# Likewise for the uniform distribution
runif(5,0,1)     # 5 trials with a uniform variable
runif(5,0,100)

#---
# rnorm, dnorm, pnorm, qnorm
# rnorm generates random values
# rnorm(n, mu, sigma) gives us n random values from Norm(mu, sigma^2)
rnorm(5,0,1)
# Test the .68, .95 rules
x = rnorm(10000,0,1)
mean(abs(x) < 1) # Since x is a vector with 10000 entries, "abs(x) < 1" is a vector of
                 # 10000 TRUE or FALSE booleans. "mean" calculates the fraction that are true.
mean(abs(x) < 2)

#---
# dnorm is the pdf; it gives the values of the density at x
x = 0
dnorm(x,0,1)
x = c(0,.2,.4,.6,.8,1.0)
dnorm(x,0,1)
dnorm(x,0,2)
# Plot the standard normal density "BELL-SHAPED CURVE"
z = seq(-6,6,.01)  #sequence from -6 to 6 in steps of .1
plot(z,dnorm(z,0,1), type='l', col='blue', lwd=3)
# Add some other densities
lines(z,dnorm(z,1,1), col='red', lwd=3) #SHIFT _1_ unit to the right (mean _1_ instead of 0)
lines(z,dnorm(z,0,2), col='green', lwd=3) #FLATTEN: increase the variance 

#---
# pnorm is the cdf
x = 0
pnorm(x,0,1)
x = c(0,.2,.4,.6,.8,1.0)
pnorm(x,0,1)
pnorm(x,0,2)
# Plot the standard normal cdf
z = seq(-6,6,.01)  #sequence from -6 to 6 in steps of .1
plot(z,pnorm(z,0,1), type='l', col='blue', lwd=3)
# Add some other cdf's
lines(z,pnorm(z,1,1), col='red', lwd=3)
lines(z,pnorm(z,0,2), col='green', lwd=3)

#---
# qnorm finds quantiles.
# It is the inverse of the cdf it takes a probability and produces a value
#a = qnorm(p, 0,1)  <==> P(Z<a) = p <==> p = pnorm(a,0,1)
qnorm(.5,0,1)
p = pnorm(.351,0,1)
p
a = qnorm(p,0,1)
a

#---
# Similarly there is runif(x,a,b) and rexp(x, lambda)
# (Likewise dunif, punif, qunif, dexp, pexp, qexp)

# Plot the pdf of exponential(.5)
x = seq(0,10,.01)
plot(x,dexp(x,.5), type='l', col='blue', lwd=3)
abline(v=0) #"abline" adds a line; "v=?" adds vertical line x=?
abline(h=0) # "h=?" adds horizontal line y=?

#------------------------------
#Histograms
# R makes plotting histograms of data easy

# Useless aside: histogram comes from the Greek word _histos_, which
# meant anything set upright: in particular, the masts of ship, hence
# (Karl Pearson, late 19th century) _histogram_.

# The same word referred to the vertical strands of fiber in a
# loom. In that meaning it came to refer (early 19th century) to
# tissue; hence _histology_.

# Modern Scientists waste less time on classical languages. If
# histograms were being named today, they would be called
# IkeaShelfPix.

# Generate some random exponential data and plot a histogram
data = rexp(100,.5)
hist(data)

# Controlling the number of bins
# breaks = a single number gives the number of bins.
# (Actually this just 'suggests' the number of bins.
# R chooses the exact number so that the bin widths are
# 1, 2, or 5 times a power of ten.)

data = rexp(100,.5)
# Run these one at a time in RStudio
hist(data, breaks=12, col='blue')
hist(data, breaks=100,col='orange')
hist(data, breaks=2, col='cyan')

# Breaks = a vector which gives the edges of each bin
data = rexp(100,.5)
# Make bins of width 0.4 that start at the minimum
# data value and go to the maximum data
binwidth = .4
bins = seq(min(data), max(data)+binwidth, binwidth)
hist(data, breaks=bins, col='purple')

#By default you get a frequency histogram.
#To get a density you set freq=FALSE
data = rexp(100,.5)
binwidth = .4
bins = seq(min(data), max(data)+binwidth, binwidth)
hist(data, breaks=bins, col='purple', freq=FALSE)
# Notice that the vertical axis is now labeled 'Density'

#With enough data the density histogram should look like the pdf
lambda = .5
data = rexp(10000,lambda)
binwidth = .3
bins = seq(min(data), max(data)+binwidth, binwidth)
hist(data, breaks=bins, col='green', freq=FALSE)
x = seq(0,max(data)+1,.01)
lines(x,dexp(x,lambda), col='purple', lwd=2)
